"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const isAssignableTo_1 = require("../Utils/isAssignableTo");
const narrowType_1 = require("../Utils/narrowType");
class ConditionalTypeNodeParser {
    constructor(typeChecker, childNodeParser) {
        this.typeChecker = typeChecker;
        this.childNodeParser = childNodeParser;
    }
    supportsNode(node) {
        return node.kind === ts.SyntaxKind.ConditionalType;
    }
    createType(node, context) {
        const checkType = this.childNodeParser.createType(node.checkType, context);
        const extendsType = this.childNodeParser.createType(node.extendsType, context);
        const result = isAssignableTo_1.isAssignableTo(extendsType, checkType);
        const tsResultType = result ? node.trueType : node.falseType;
        const resultType = this.childNodeParser.createType(tsResultType, context);
        const checkTypeParameterName = this.getTypeParameterName(node.checkType);
        const resultTypeParameterName = this.getTypeParameterName(tsResultType);
        if (resultTypeParameterName != null && resultTypeParameterName === checkTypeParameterName) {
            return narrowType_1.narrowType(resultType, type => isAssignableTo_1.isAssignableTo(extendsType, type) === result);
        }
        return resultType;
    }
    getTypeParameterName(node) {
        if (ts.isTypeReferenceNode(node)) {
            const typeSymbol = this.typeChecker.getSymbolAtLocation(node.typeName);
            if (typeSymbol.flags & ts.SymbolFlags.TypeParameter) {
                return typeSymbol.name;
            }
        }
        return null;
    }
}
exports.ConditionalTypeNodeParser = ConditionalTypeNodeParser;
//# sourceMappingURL=ConditionalTypeNodeParser.js.map